/****************************************************************
 *      delete_lists.c
 *      
 *      Delete the 16 segment lists used to record the cells
 *      that are visible from each viewpoint.
 *
 ****************************************************************/

/* Modified from r.cumlos
 * by MWL on 1/3/96 to:-
 * 
 * Updated by Mark Lake on 17/8/01 for GRASS 5.x
 * 
 * 1) Delete the 16 segment lists used to record the cells
 *    that are visible from each viewpoint.
 *
 * Modifications to r.cumlos version 3.0:-
 *
 * 1) Taken out of main() and placed in function delete_lists().
 *
 * Modifications to r.los:-
 *
 * 1) r.los does not perform this function.
 *
 * Called by:-
 * 
 * 1) scan_all_cells()
 * 2) random_sample()
 * 3) systematic_sample()
 * 4) from_sites()
 *
 * Calls:-
 *
 * 1)  delete_one_list()
 */

#include <grass/segment.h>

#include "config.h"
#include "point.h"


/* struct point *heads[] is same as "struct point **heads" */
void delete_lists (struct point *heads[])
{
	int segment_no;
	void delete_one_list ();
	int i;

	/* delete the list of visible cells */
	for (segment_no = 1; segment_no <= 16; segment_no++)
	{
		delete_one_list (heads[segment_no-1]);
	}

	/* release pointers */
	for (i = 0; i < delete_lists_get_next(); i++) {
		if ( DELETION_LIST[i] != NULL ) {
			G_free(DELETION_LIST[i]);
			DELETION_LIST[i] = NULL;
		}
	}
}


void delete_lists_init ()
{

	DELETION_LIST=G_malloc(sizeof(struct point*)*MAX_POINTS);

	int i = 0;
	for ( i = 0; i < MAX_POINTS; i ++ ) {
		DELETION_LIST[i] = NULL;
	}

	DELETION_LIST_NEXT = 0;
}


void delete_lists_depose ()
{
	int i = 0;
	for ( i = 0; i < MAX_POINTS; i ++ ) {
		if ( DELETION_LIST[i] != NULL ) {
			G_free ( DELETION_LIST[i] );
		}		
	}

	G_free ( DELETION_LIST );
}


void delete_lists_clear ()
{
	int i = 0;
	for ( i = 0; i < MAX_POINTS; i ++ ) {
		if ( DELETION_LIST[i] != NULL ) {
			G_free ( DELETION_LIST[i] );
			DELETION_LIST[i] = NULL;	
		}		
	}
}


void delete_lists_rewind ()
{
	DELETION_LIST_NEXT = 0;
}


int delete_lists_get_next ()
{
	return ( DELETION_LIST_NEXT );
}




